=== modified file '.bzrignore'
--- .bzrignore	2010-04-14 06:34:41 +0000
+++ .bzrignore	2010-04-22 08:52:59 +0000
@@ -8,3 +8,4 @@
 _trial_temp
 loggerhead-memprofile
 ./docs/_build/
+tags

=== modified file 'NEWS'
--- NEWS	2010-04-18 17:57:17 +0000
+++ NEWS	2010-05-13 22:18:07 +0000
@@ -4,6 +4,13 @@
 dev [future]
 ------------
 
+    - Remove ``start-loggerhead`` and ``stop-loggerhead`` which were already
+      deprecated. (John Arbash Meinel)
+
+    - Syntax highlighting is no longer applied for files greater than 512K,
+      reducing codebrowse.launchpad.net overloading.
+      (Max Kanat-Alexander, #513044)
+
     - Documentation added in the docs directory. README simplified
       accordingly. (Tres Seaver).
 
@@ -21,8 +28,15 @@
     - Ignore readonly+ prefix when checking if Loggerhead is serving a
       local location. (Reported by Tres Seaver.) (Matt Nordhoff)
 
-    - Add a script and documentation for running under mod_wsgi.
-      (Stuart Colville, Toshio Kuratomi)
+    - Set Cache-Control and Expires headers on static pages.
+      (John Arbash Meinel)
+
+    - Generate relative links where possible (everywhere but HTTP
+      redirects and feed IDs). (Michael Hudson, Matt Nordhoff)
+
+    - Fix bad redirect when visiting "/download" or "/download/".
+      (Matt Nordhoff, #247992)
+
 
 1.17 [20Aug2009]
 ----------------

=== modified file '__init__.py'
--- __init__.py	2010-04-13 23:35:01 +0000
+++ __init__.py	2010-05-10 19:36:37 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009 Canonical Ltd
+# Copyright 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -58,18 +58,34 @@
     HELP = ('Loggerhead, a web-based code viewer and server. (default port: %d)' %
             (DEFAULT_PORT,))
 
-    def serve_http(transport, host=None, port=None, inet=None):
-        from paste.httpexceptions import HTTPExceptionHandler
-        from paste.httpserver import serve
-
+    def setup_logging(config):
+        import logging
+        import sys
+
+        logger = logging.getLogger('loggerhead')
+        handler = logging.StreamHandler(sys.stderr)
+        handler.setLevel(logging.DEBUG)
+        logger.addHandler(handler)
+        logging.getLogger('simpleTAL').addHandler(handler)
+        logging.getLogger('simpleTALES').addHandler(handler)
+
+
+    def _ensure_loggerhead_path():
+        """Ensure that you can 'import loggerhead' and get the root."""
         # loggerhead internal code will try to 'import loggerhead', so
         # let's put it on the path if we can't find it in the existing path
         try:
-            import loggerhead
+            import loggerhead.apps.transport
         except ImportError:
             import os.path, sys
             sys.path.append(os.path.dirname(__file__))
 
+    def serve_http(transport, host=None, port=None, inet=None):
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+
+        _ensure_loggerhead_path()
+
         from loggerhead.apps.transport import BranchesFromTransportRoot
         from loggerhead.config import LoggerheadConfig
 
@@ -81,6 +97,7 @@
         if not transport.is_readonly():
             argv.insert(0, '--allow-writes')
         config = LoggerheadConfig(argv)
+        setup_logging(config)
         app = BranchesFromTransportRoot(transport.base, config)
         app = HTTPExceptionHandler(app)
         serve(app, host=host, port=port)
@@ -120,3 +137,9 @@
                     super(cmd_serve, self).run(*args, **kw)
 
         register_command(cmd_serve)
+
+    def load_tests(standard_tests, module, loader):
+        _ensure_loggerhead_path()
+        standard_tests.addTests(loader.loadTestsFromModuleNames(
+            ['bzrlib.plugins.loggerhead.loggerhead.tests']))
+        return standard_tests

=== removed file 'apache-loggerhead.conf'
--- apache-loggerhead.conf	2010-04-18 17:55:33 +0000
+++ apache-loggerhead.conf	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-### If you receive MemoryError tracebacks setting up loggerhead under mod_wsgi,
-### read /usr/share/doc/loggerhead*/README for help
-Alias /bzr/static /usr/share/loggerhead/static
-RewriteEngine On
-RewriteRule ^/bzr$ /bzr/ [R]
-
-WSGIDaemonProcess loggerhead user=apache group=apache maximum-requests=1000 display-name=loggerhead processes=4 threads=1
-WSGISocketPrefix run/wsgi
-WSGIRestrictStdout On
-WSGIRestrictSignal Off
-
-WSGIScriptAlias /bzr /usr/bin/loggerhead.wsgi
-
-<Location /bzr>
-    WSGIProcessGroup loggerhead
-    Order deny,allow
-    Allow from all
-</Location>
-

=== removed file 'bazaar.conf'
--- bazaar.conf	2010-04-18 14:41:23 +0000
+++ bazaar.conf	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-# directory to serve bzr branches from
-# Non-bzr directories under this path will also be visible in loggerhead
-#http_root_dir = '/var/www/bzr'
-
-# The url prefix for the bzr branches.
-http_user_prefix = '/bzr'
-
-# Directory to put cache files in
-http_sql_dir = '/var/cache/loggerhead'

=== modified file 'docs/index.rst'
--- docs/index.rst	2010-04-18 17:55:33 +0000
+++ docs/index.rst	2010-04-13 08:46:25 +0000
@@ -149,61 +149,6 @@
 some path into the site, you'll need to specify it using
 ``--prefix=/some_path``.
 
-Serving Loggerhead with mod_wsgi
---------------------------------
-
-A second method for using loggehead with apache is to have apache itself
-execute loggerhead via mod_wsgi.  You need to add configuration for apache and
-for bazaar to make this work.  Example config files are in the loggerhead doc
-directory as apache-loggerhead.conf and bazaar.conf.  You can copy them into
-place and use them as a starting point following these directions:
-
-1) Install mod_wsgi.  On Ubuntu and other Debian derived distros::
-
-    sudo apt-get install libapache2-mod-wsgi
-
-   On Fedora-derived distros::
-
-    su -c yum install mod_wsgi
-
-2) Copy the bazaar conf file where apache will find it (May be done for you if
-   you installed loggerhead from a distribution package)::
-
-    # install -d -o apache -g apache -m 0755 /etc/loggerhead
-    # cp -p /usr/share/doc/loggerhead*/bazaar.conf /etc/loggerhead/
-    # ln -s /etc/loggerhead /var/www/.bazaar
-
-3) Create the cache directory (May be done for you if you installed loggerhead
-   from a distribution package)::
-
-    # install -d -o apache -g apache -m 0700 /var/cache/loggerhead/
-
-4) Edit /etc/loggerhead/bazaar.conf.  You need to set http_root_dir to the filesystem
-   path that you will find your bzr branches under.  Note that normal
-   directories under that path will also be visible in loggerhead.
-
-5) Install the apache conf file::
-
-     # cp -p /usr/share/doc/loggerhead*/apache-loggerhead.conf /etc/httpd/conf.d/loggerhead.conf
-
-6) Edit /etc/httpd/conf.d/loggerhead.conf to point to the url you desire to
-   serve loggerhead on.  This should match with the setting for
-   http_user_prefix in bazaar.conf
-
-7) Restart apache and you should be able to start browsing
-
-.. note:: If you have SELinux enabled on your system you may need to allow
-   apache to execute files in temporary directories.  You will get a
-   MemoryError traceback from python if this is the case.  This is because of
-   the way that python ctypes interacts with libffi.  To rectify this, you may
-   have to do several things, such as mounting tmpdirs so programs can be
-   executed on them and setting this SELinux boolean::
-
-       setsebool httpd_tmp_exec on
-
-   This bug has information about how python and/or Linux distros might solve
-   this issue permanently and links to bugs which diagnose the root cause.
-   https://bugzilla.redhat.com/show_bug.cgi?id=582009
 
 Search
 ------

=== removed file 'docs/start-loggerhead.rst'
--- docs/start-loggerhead.rst	2010-03-25 10:39:30 +0000
+++ docs/start-loggerhead.rst	1970-01-01 00:00:00 +0000
@@ -1,46 +0,0 @@
-:command:`start-loggerhead`
-===========================
-
-The :command:`start-loggerhead` command starts a new standalone Loggerhead
-server.  By default, the server runs in the background (daemonized).
-
-.. program:: start-loggerhead
-
-Usage
------
-
-.. code-block:: sh
-
-    start-loggerhead [OPTIONS]
-
-Options
--------
-
-.. cmdoption:: --version
-
-    Print the software version and exit.
-
-.. cmdoption:: -h, --help
-
-    Show this help message and exit.
-
-.. cmdoption:: --foreground
-
-    Run in the foreground;  don't daemonize.
-
-.. cmdoption:: -C, --check
-
-    Only start if not already running (useful for cron jobs).
-
-.. cmdoption:: -p, --pidfile=PIDFILE
-
-    Override pid file location.
-
-.. cmdoption:: -c, --config-file=CONFIGFILE
-
-    Override configuration file location (the default is
-    :file:`/etc/loggerhead.conf`).
-
-.. cmdoption:: --log-folder=LOG_FOLDER
-
-    The directory in which to place log files.

=== removed file 'docs/stop-loggerhead.rst'
--- docs/stop-loggerhead.rst	2010-03-25 10:25:13 +0000
+++ docs/stop-loggerhead.rst	1970-01-01 00:00:00 +0000
@@ -1,18 +0,0 @@
-:command:`stop-loggerhead`
-======================================================================
-
-The :command:`stop-loggerhead` command stops the currently running Loggerhead
-daemon.
-
-.. program:: stop-loggerhead
-
-Usage
------
-
-.. code-block:: sh
-
-   stop-loggerhead [OPTIONS]
-
-.. cmdoption:: -p, --pidfile=PIDFILE
-
-   override pid file location

=== removed file 'loggerhead.wsgi'
--- loggerhead.wsgi	2010-04-18 17:58:04 +0000
+++ loggerhead.wsgi	1970-01-01 00:00:00 +0000
@@ -1,50 +0,0 @@
-#!/usr/bin/python -tt
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-import sys
-import os
-import pwd
-sys.path.insert(0, os.path.dirname(__file__))
-
-from paste.httpexceptions import HTTPExceptionHandler
-from loggerhead.apps.transport import BranchesFromTransportRoot
-from loggerhead.apps.error import ErrorHandlerApp
-from loggerhead.config import LoggerheadConfig
-from bzrlib import config as bzrconfig
-from paste.deploy.config import PrefixMiddleware
-from bzrlib.plugin import load_plugins
-
-class NotConfiguredError(Exception):
-    pass
-
-
-load_plugins()
-config = LoggerheadConfig()
-prefix = config.get_option('user_prefix') or ''
-# Note we could use LoggerheadConfig here if it didn't fail when a
-# config option is not also a commandline option
-root_dir = bzrconfig.GlobalConfig().get_user_option('http_root_dir')
-if not root_dir:
-    raise NotConfiguredError('You must have a ~/.bazaar/bazaar.conf file for'
-            ' %(user)s with http_root_dir set to the base directory you want'
-            ' to serve bazaar repositories from' %
-            {'user': pwd.getpwuid(os.geteuid()).pw_name})
-prefix = prefix.encode('utf-8', 'ignore')
-root_dir = root_dir.encode('utf-8', 'ignore')
-app = BranchesFromTransportRoot(root_dir, config)
-app = PrefixMiddleware(app, prefix=prefix)
-app = HTTPExceptionHandler(app)
-application = ErrorHandlerApp(app)

=== modified file 'loggerhead/apps/__init__.py'
--- loggerhead/apps/__init__.py	2009-11-26 03:44:40 +0000
+++ loggerhead/apps/__init__.py	2010-04-30 21:30:18 +0000
@@ -9,7 +9,9 @@
 static = os.path.join(
     os.path.dirname(os.path.dirname(__file__)), 'static')
 
-static_app = urlparser.make_static(None, static)
+# Static things can be cached for half a day, we could probably make this
+# longer, except for just before rollout times.
+static_app = urlparser.make_static(None, static, cache_max_age=12*60*60)
 
 favicon_app = convert_file_errors(fileapp.FileApp(
     os.path.join(static, 'images', 'favicon.ico')))

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2010-03-25 16:19:24 +0000
+++ loggerhead/apps/branch.py	2010-05-05 18:28:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -89,10 +89,15 @@
             if v is not None:
                 qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
-        return request.construct_url(
-            self._environ, script_name=self._url_base,
-            path_info=unicode('/'.join(args)).encode('utf-8'),
-            querystring=qs)
+        path_info = urllib.quote(
+            unicode('/'.join(args)).encode('utf-8'), safe='/~:')
+        if qs:
+            path_info += '?' + qs
+        return self._url_base + path_info
+
+    def absolute_url(self, *args, **kw):
+        rel_url = self.url(*args, **kw)
+        return request.resolve_relative_url(rel_url, self._environ)
 
     def context_url(self, *args, **kw):
         kw = util.get_context(**kw)
@@ -154,7 +159,7 @@
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
-                self._url_base + '/changes')
+                self.absolute_url('/changes'))
         if path == 'static':
             return static_app(environ, start_response)
         cls = self.controllers_dict.get(path)

=== modified file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2009-05-13 13:14:04 +0000
+++ loggerhead/apps/config.py	2010-05-05 18:35:35 +0000
@@ -208,8 +208,8 @@
                                 environ['SCRIPT_NAME']
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             response = WSGIResponse()
             self.browse(response)

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2010-03-25 16:19:24 +0000
+++ loggerhead/apps/transport.py	2010-05-05 18:35:35 +0000
@@ -33,6 +33,7 @@
 from loggerhead.apps import favicon_app, robots_app, static_app
 from loggerhead.controllers.directory_ui import DirectoryUI
 
+# TODO: Use bzrlib.ui.bool_from_string(), added in bzr 1.18
 _bools = {
     'yes': True, 'no': False,
     'on': True, 'off': False,
@@ -68,8 +69,8 @@
     def app_for_non_branch(self, environ):
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             if self.name:
                 name = self.name

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2010-04-08 01:08:33 +0000
+++ loggerhead/controllers/download_ui.py	2010-05-05 19:03:40 +0000
@@ -44,8 +44,8 @@
             args.append(arg)
 
         if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url(
-                      '../changes'))
+            raise httpexceptions.HTTPMovedPermanently(
+                self._branch.absolute_url('/changes'))
 
         revid = h.fix_revid(args[0])
         file_id = args[1]

=== modified file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	2009-05-16 06:03:47 +0000
+++ loggerhead/highlight.py	2010-04-22 03:23:55 +0000
@@ -16,6 +16,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import cgi
+
 from pygments import highlight as _highlight_func
 from pygments.lexers import guess_lexer, guess_lexer_for_filename, TextLexer
 from pygments.formatters import HtmlFormatter
@@ -23,12 +25,19 @@
 
 DEFAULT_PYGMENT_STYLE = 'colorful'
 
+# Trying to highlight very large files using pygments was killing
+# loggerhead on launchpad.net, because pygments isn't very fast.
+# So we only highlight files if they're 512K or smaller.
+MAX_HIGHLIGHT_SIZE = 512000;
 
 def highlight(path, text, encoding, style=DEFAULT_PYGMENT_STYLE):
     """
     Returns a list of highlighted (i.e. HTML formatted) strings.
     """
 
+    if len(text) > MAX_HIGHLIGHT_SIZE:
+        return map(cgi.escape, text.split('\n'))
+
     formatter = HtmlFormatter(style=style, nowrap=True, classprefix='pyg-')
 
     try:

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2010-02-26 04:37:13 +0000
+++ loggerhead/history.py	2010-05-04 02:22:15 +0000
@@ -152,6 +152,9 @@
                 filename=rich_filename(paths[1], kind),
                 file_id=file_id))
 
+# The lru_cache is not thread-safe, so we need a lock around it for
+# all threads.
+rev_info_memory_cache_lock = threading.RLock()
 
 class RevInfoMemoryCache(object):
     """A store that validates values against the revids they were stored with.
@@ -175,7 +178,11 @@
         If a value was stored under `key`, with the same revid, return it.
         Otherwise return None.
         """
-        cached = self._cache.get(key)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            cached = self._cache.get(key)
+        finally:
+            rev_info_memory_cache_lock.release()
         if cached is None:
             return None
         stored_revid, data = cached
@@ -187,7 +194,11 @@
     def set(self, key, revid, data):
         """Store `data` under `key`, to be checked against `revid` on get().
         """
-        self._cache[key] = (revid, data)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            self._cache[key] = (revid, data)
+        finally:
+            rev_info_memory_cache_lock.release()
 
 # Used to store locks that prevent multiple threads from building a 
 # revision graph for the same branch at the same time, because that can
@@ -283,6 +294,7 @@
         else:
             self._file_change_cache = None
         self._branch = branch
+        self._branch_tags = None
         self._inventory_cache = {}
         self._branch_nick = self._branch.get_config().get_nickname()
         self.log = logging.getLogger('loggerhead.%s' % (self._branch_nick,))
@@ -653,11 +665,12 @@
         """
         message, short_message = clean_message(revision.message)
 
-        tags = self._branch.tags.get_reverse_tag_dict()
+        if self._branch_tags is None:
+            self._branch_tags = self._branch.tags.get_reverse_tag_dict()
 
         revtags = None
-        if tags.has_key(revision.revision_id):
-          revtags = ', '.join(tags[revision.revision_id])
+        if revision.revision_id in self._branch_tags:
+          revtags = ', '.join(self._branch_tags[revision.revision_id])
 
         entry = {
             'revid': revision.revision_id,

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2009-06-06 12:23:11 +0000
+++ loggerhead/templates/atom.pt	2010-05-05 18:42:58 +0000
@@ -4,8 +4,8 @@
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
   <updated tal:content="string:${updated}Z">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.url(['/atom'])" />
+  <id tal:content="python:branch.absolute_url(['/atom'])">url</id>
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.absolute_url(['/atom'])" />
   <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
@@ -16,7 +16,7 @@
       updated
     </updated>
     <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <id tal:content="python:branch.absolute_url(['/revision', entry.revno])">
       ID
     </id>
     <author tal:repeat="author entry/authors">

=== modified file 'loggerhead/tests/__init__.py'
--- loggerhead/tests/__init__.py	2006-12-11 06:44:19 +0000
+++ loggerhead/tests/__init__.py	2010-05-10 19:36:37 +0000
@@ -0,0 +1,26 @@
+# Copyright 2006, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+def load_tests(standard_tests, module, loader):
+    standard_tests.addTests(loader.loadTestsFromModuleNames([
+        (__name__ + '.' + x) for x in [
+            'test_controllers',
+            'test_corners',
+            'test_simple',
+            'test_templating',
+        ]]))
+    return standard_tests

=== removed file 'loggerhead/trace.py'
--- loggerhead/trace.py	2008-10-24 02:26:05 +0000
+++ loggerhead/trace.py	1970-01-01 00:00:00 +0000
@@ -1,75 +0,0 @@
-#
-# Copyright (C) 2008  Canonical Ltd.
-#                     (Authored by Martin Albisetti <argentina@gmail.com)
-# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
-# Copyright (C) 2006  Robey Pointer <robey@lag.net>
-# Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
-
-import os
-import logging
-import sys
-
-
-def make_handler(config, filename):
-    roll = config.get('log.roll', 'never')
-    if roll == 'daily':
-        h = logging.handlers.TimedRotatingFileHandler(filename,
-                                                      'midnight',
-                                                      1,
-                                                      100)
-    elif roll == 'weekly':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
-    else:
-        h = logging.FileHandler(filename)
-    return h
-
-
-def setup_logging(log_folder, config, foreground):
-    # i hate that stupid logging config format, so just set up logging here.
-
-    if not os.path.exists(log_folder):
-        os.mkdir(log_folder)
-
-    f = logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d]'
-                          ' %(name)s: %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    debug_log = make_handler(config, os.path.join(log_folder, 'debug.log'))
-    debug_log.setLevel(logging.DEBUG)
-    debug_log.setFormatter(f)
-    if foreground:
-        stdout_log = logging.StreamHandler(sys.stdout)
-        stdout_log.setLevel(logging.DEBUG)
-        stdout_log.setFormatter(f)
-    f = logging.Formatter('[%(asctime)s.%(msecs)03d] %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    access_log = make_handler(config, os.path.join(log_folder, 'access.log'))
-    access_log.setLevel(logging.INFO)
-    access_log.setFormatter(f)
-
-    error_log = make_handler(config, os.path.join(log_folder, 'error.log'))
-    error_log.setLevel(logging.ERROR)
-    error_log.setFormatter(f)
-
-    logging.getLogger('').setLevel(logging.DEBUG)
-    logging.getLogger('').addHandler(debug_log)
-    logging.getLogger('loggerhead.access').addHandler(access_log)
-    logging.getLogger('loggerhead').setLevel(logging.ERROR)
-    logging.getLogger('loggerhead').addHandler(error_log)
-
-    if foreground:
-        logging.getLogger('').addHandler(stdout_log)

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2010-03-25 16:19:24 +0000
+++ loggerhead/util.py	2010-04-24 12:40:17 +0000
@@ -68,7 +68,7 @@
 
 def date_time(value):
     if value is not None:
-        return value.strftime('%Y-%m-%d %T')
+        return value.strftime('%Y-%m-%d %H:%M:%S')
     else:
         return 'N/A'
 
@@ -137,6 +137,7 @@
     """
 
     def __init__(self, _dict=None, **kw):
+        self._properties = {}
         if _dict is not None:
             for key, value in _dict.iteritems():
                 setattr(self, key, value)
@@ -153,6 +154,24 @@
         out += '}'
         return out
 
+    def __getattr__(self, attr):
+        """Used for handling things that aren't already available."""
+        if attr.startswith('_') or attr not in self._properties:
+            raise AttributeError('No attribute: %s' % (attr,))
+        val = self._properties[attr](self, attr)
+        setattr(self, attr, val)
+        return val
+
+    def _set_property(self, attr, prop_func):
+        """Set a function that will be called when an attribute is desired.
+
+        We will cache the return value, so the function call should be
+        idempotent. We will pass 'self' and the 'attr' name when triggered.
+        """
+        if attr.startswith('_'):
+            raise ValueError("Cannot create properties that start with _")
+        self._properties[attr] = prop_func
+
 
 def trunc(text, limit=10):
     if len(text) <= limit:

=== modified file 'setup.py'
--- setup.py	2010-04-18 17:40:53 +0000
+++ setup.py	2010-05-13 22:18:07 +0000
@@ -30,10 +30,7 @@
     license = "GNU GPL v2 or later",
     maintainer = "Michael Hudson",
     maintainer_email = "michael.hudson@canonical.com",
-    scripts = ["start-loggerhead",
-               "stop-loggerhead",
-               "serve-branches",
-               "loggerhead.wsgi"],
+    scripts = ["serve-branches"],
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
@@ -67,12 +64,7 @@
                                    "static/javascript/yui/build/yui-base/*",
                                    "static/images/*"]},
     data_files = [
-        ('share/man/man1', ['start-loggerhead.1',
-                            'stop-loggerhead.1',
-                            'serve-branches.1']),
-        ('share/doc/loggerhead', ['loggerhead.conf.example',
-                                  'README.mod_wsgi',
-                                  'apache-loggerhead.conf',
-                                  'bazaar.conf']),
+        ('share/man/man1', ['serve-branches.1']),
+        ('share/doc/loggerhead', ['loggerhead.conf.example']),
         ],
     )

=== removed file 'start-loggerhead'
--- start-loggerhead	2009-10-17 06:35:33 +0000
+++ start-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,146 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""
-WARNING! This script has been deprecated and will go away soon!
-A script for starting the loggerhead process.
-"""
-
-
-import logging
-from optparse import OptionParser
-import os
-import sys
-import urlparse
-
-from bzrlib.plugin import load_plugins
-from bzrlib.util.configobj.configobj import ConfigObj
-
-from paste import httpserver
-from paste.httpexceptions import make_middleware
-from paste.translogger import make_filter
-
-from loggerhead import daemon
-from loggerhead.apps.config import Root
-from loggerhead.trace import setup_logging
-from loggerhead.apps.error import ErrorHandlerApp
-
-
-def main():
-    sys.stderr.write('\n\n')
-    sys.stderr.write('WARNING!!! This script has been deprecated by '
-                     'serve-branches, and will be removed in the next '
-                     'release. Please migrate to serve-branches and report '
-                     'any missing features.\n')
-    sys.stderr.write('\n\n')
-
-    home = os.path.realpath(os.path.dirname(__file__))
-    default_pidfile = os.path.join(home, 'loggerhead.pid')
-    default_configfile = os.path.join(home, 'loggerhead.conf')
-    default_log_folder = os.path.join(home, 'logs')
-    parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-    parser.add_option('-f', '--foreground',
-                      action='store_true', dest='foreground', default=False,
-                      help="run in the foreground; don't daemonize")
-    parser.add_option('-C', '--check', action='store_true',
-                      dest='check', default=False,
-                      help=("only start if not already running (useful for "
-                            "cron jobs)"))
-    parser.add_option('-p', '--pidfile', dest="pidfile",
-                      default=default_pidfile,
-                      type=str, help="override pidfile location")
-    parser.add_option('-c', '--config-file', dest="configfile",
-                      default=default_configfile,
-                      type=str, help="override configuration file location")
-    parser.add_option('-L', '--log-folder', dest="log_folder",
-                      default=default_log_folder,
-                      type=str, help="override log file directory")
-    options, args = parser.parse_args()
-    if len(args) > 0:
-        parser.error('No filename arguments are used, only options.')
-
-    if options.check:
-        if daemon.is_running(options.pidfile):
-            sys.exit(0)
-        sys.stderr.write('Did not find loggerhead running in %r;' % (
-                         options.pidfile))
-        sys.stderr.write(' restarting...\n')
-
-    # read loggerhead config
-
-    config = ConfigObj(options.configfile, encoding='utf-8')
-    extra_path = config.get('bzrpath', None)
-    if extra_path:
-        sys.path.insert(0, extra_path)
-
-    potential_overrides = [('server.socket_port', int),
-                           ('server.webpath', str),
-                           ('server.thread_pool', int),
-                           ('server.socket_host', str)]
-    server_port = int(config.get('server.socket_port', 8080))
-    nworkers = int(config.get('server.thread_pool', 10))
-    server_host = config.get('server.socket_host', '0.0.0.0')
-    webpath = config.get('server.webpath', None)
-
-    for key, keytype in potential_overrides:
-        value = config.get(key, None)
-        if value is not None:
-            value = keytype(value)
-
-    if not options.foreground:
-        sys.stderr.write('\n')
-        sys.stderr.write('Launching loggerhead into the background.\n')
-        sys.stderr.write('PID file: %s\n' % (options.pidfile,))
-        sys.stderr.write('\n')
-
-        daemon.daemonize(options.pidfile, home)
-
-    setup_logging(options.log_folder, config, foreground=options.foreground)
-
-    log = logging.getLogger('loggerhead')
-    log.info('Starting up...')
-
-    app = Root(config)
-
-    app = app
-    app = make_middleware(app)
-    app = make_filter(app, None, logger_name=log.name+'.access')
-    app = ErrorHandlerApp(app)
-
-    if webpath:
-        scheme, netloc, path, blah, blah, blah = urlparse.urlparse(webpath)
-
-        def app(environ, start_response, orig=app):
-            environ['SCRIPT_NAME'] = path
-            environ['HTTP_HOST'] = netloc
-            environ['wsgi.url_scheme'] = scheme
-            return orig(environ, start_response)
-
-    load_plugins()
-
-    try:
-        httpserver.serve(
-            app, host=server_host, port=server_port,
-            threadpool_workers=nworkers)
-    finally:
-        log.info('Shutdown.')
-        try:
-            os.remove(options.pidfile)
-        except OSError:
-            pass
-
-
-if __name__ == '__main__':
-    main()

=== removed file 'start-loggerhead.1'
--- start-loggerhead.1	2008-08-06 18:33:20 +0000
+++ start-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,38 +0,0 @@
-.TH START-LOGGERHEAD "1" "July 2008" "start-loggerhead 1.2.1" "User Commands"
-.SH NAME
-start-loggerhead \- start a loggerhead server
-.SH SYNOPSIS
-.B start-loggerhead
-[\fIoptions\fR]
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.PP
-The start-loggerhead command starts a new standalone loggerhead server.
-.SH OPTIONS
-.TP
-\fB\-\-version\fR
-show program's version number and exit
-.TP
-\fB\-h\fR, \fB\-\-help\fR
-show this help message and exit
-.TP
-\fB\-f\fR, \fB\-\-foreground\fR
-run in the foreground; don't daemonize
-.TP
-\fB\-C\fR, \fB\-\-check\fR
-only start if not already running (useful for cron jobs)
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.TP
-\fB\-c\fR, \fB\-\-config-file=CONFIGFILE\fR
-override configuration file location
-.TP
-\fB\-L\fR, \fB\-\-log-folder=LOG-FOLDER\fR
-override log directory
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

=== removed file 'stop-loggerhead'
--- stop-loggerhead	2009-05-13 14:03:48 +0000
+++ stop-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,51 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""A script for stopping the loggerhead process."""
-
-import os
-import sys
-from optparse import OptionParser
-
-home = os.path.realpath(os.path.dirname(__file__))
-default_pidfile = os.path.join(home, 'loggerhead.pid')
-parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-parser.add_option('-p', '--pidfile', dest="pidfile", default=default_pidfile,
-                  help="override pidfile location")
-
-options, args = parser.parse_args()
-if len(args) > 0:
-    parser.error('No filename arguments are used, only options.')
-
-try:
-    f = open(options.pidfile, 'r')
-except IOError, e:
-    print 'Pid file %s not found.' % (options.pidfile,)
-    sys.exit(1)
-
-pid = int(f.readline())
-
-try:
-    os.kill(pid, 0)
-except OSError, e:
-    print 'Stale pid file; server is not running.'
-    sys.exit(1)
-
-print
-print 'Shutting down previous server @ pid %d.' % (pid,)
-print
-
-import signal
-os.kill(pid, signal.SIGINT)

=== removed file 'stop-loggerhead.1'
--- stop-loggerhead.1	2008-08-06 18:39:15 +0000
+++ stop-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-.TH STOP-LOGGERHEAD "1" "July 2008" "stop-loggerhead 1.2.1" "User Commands"
-.SH NAME
-stop-loggerhead \- stop a loggerhead server
-.SH SYNOPSIS
-.B stop-loggerhead
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.SH OPTIONS
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.PP
-The stop-loggerhead command stops the currently running loggerhead server.
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

